SystemOrganization addCategory: #'Neo-Console-Core'!

Object subclass: #NeoConsoleCommand
	instanceVariableNames: 'arguments result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleCommand commentStamp: 'SvenVanCaekenberghe 2/24/2015 14:33' prior: 0!
I am NeoConsoleCommand. I describe a command that can be executed by the NeoConsole infrastructure. I am abstract.!

!NeoConsoleCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:40'!
defaultCommand
	^ NeoConsoleEvalCommand ! !

!NeoConsoleCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:44'!
forLine: line
	| commandClass |
	commandClass := self knownCommands 
		detect: [ :each | each handlesLine: line ] 
		ifNone: [ self defaultCommand ].
	^ commandClass! !

!NeoConsoleCommand class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:35'!
handlesLine: line
	^ line beginsWith: self prefix! !

!NeoConsoleCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:19'!
helpText
	^ self name, String cr, self comment! !

!NeoConsoleCommand class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:31'!
isMultiLine
	^ self isSingleLine not! !

!NeoConsoleCommand class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:31'!
isSingleLine
	^ true! !

!NeoConsoleCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:36'!
knownCommands
	^ self subclasses! !

!NeoConsoleCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:15'!
prefix
	^ self subclassResponsibility ! !

!NeoConsoleCommand methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 16:43'!
argument
	^ arguments first! !

!NeoConsoleCommand methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 14:34'!
arguments
	^ arguments ! !

!NeoConsoleCommand methodsFor: 'public' stamp: 'SvenVanCaekenberghe 2/24/2015 15:17'!
execute
	self subclassResponsibility ! !

!NeoConsoleCommand methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/24/2015 16:09'!
hasArguments
	^ arguments notNil and: [ arguments notEmpty ]! !

!NeoConsoleCommand methodsFor: 'initialization-release' stamp: 'SvenVanCaekenberghe 2/24/2015 16:29'!
on: input
	arguments := Array with: input trimBoth! !

!NeoConsoleCommand methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:38'!
result
	^ result! !

NeoConsoleCommand subclass: #NeoConsoleEvalCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleEvalCommand commentStamp: 'SvenVanCaekenberghe 2/24/2015 14:39' prior: 0!
Evaluate a Pharo expression!

!NeoConsoleEvalCommand class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:31'!
isSingleLine
	^ false! !

!NeoConsoleEvalCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:18'!
prefix
	^ 'eval'! !

!NeoConsoleEvalCommand methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/16/2015 16:32'!
execute
	result := String 
		streamContents: [ :out | 
			[ 	| compiler evaluationResult |
				compiler := self class compiler.		
				evaluationResult := compiler evaluate: self argument. 
				out print: evaluationResult ]
					on: Error, ParserNotification
					do: [ :exception |
						out print: exception; cr.
						exception signalerContext printDetails: out.
						exception signalerContext sender debugStack: 8 on: out ] ]
		limitedTo: self outputLimit.
	result size < self outputLimit 
		ifFalse: [ result := result , ' ... (truncated) ...' ]! !

!NeoConsoleEvalCommand methodsFor: 'initialization-release' stamp: 'SvenVanCaekenberghe 2/24/2015 16:50'!
on: input
	super on: input.
	(self argument beginsWith: self class prefix)
		ifTrue: [ arguments := Array with: (self argument allButFirst: self class prefix size) trimBoth ]
	! !

!NeoConsoleEvalCommand methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/16/2015 16:26'!
outputLimit
	^ 16384! !

NeoConsoleCommand subclass: #NeoConsoleGetCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleGetCommand commentStamp: 'SvenVanCaekenberghe 2/24/2015 14:40' prior: 0!
Get the value of a metric (gauge or counter)!

!NeoConsoleGetCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:18'!
prefix
	^ 'get'! !

!NeoConsoleGetCommand methodsFor: 'public' stamp: 'SvenVanCaekenberghe 2/24/2015 17:39'!
execute
	self hasArguments 
		ifTrue: [ 
			[ result := (NeoConsoleMetric named: self argument) value asString ] 
				on: NotFound 
				do: [ :exception | result := exception printString ] ]
		ifFalse: [ 
			result := String streamContents: [ :out |
				out << 'known metrics:'; cr.
				NeoConsoleMetric knownMetrics 
					do: [ :each | out space; space; << each name; space; << each description ] 
					separatedBy: [ out cr ] ] ]! !

!NeoConsoleGetCommand methodsFor: 'initialization-release' stamp: 'SvenVanCaekenberghe 2/24/2015 17:30'!
on: input
	super on: input.
	arguments := self argument findTokens: Character separators.
	arguments := arguments allButFirst ! !

NeoConsoleCommand subclass: #NeoConsoleHelpCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleHelpCommand commentStamp: 'SvenVanCaekenberghe 2/24/2015 14:40' prior: 0!
Help!

!NeoConsoleHelpCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:18'!
prefix
	^ 'help'! !

!NeoConsoleHelpCommand methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/16/2015 16:16'!
execute
	self hasArguments 
		ifTrue: [ 
			(NeoConsoleCommand knownCommands 
					detect: [ :each | each prefix = self argument ] ifNone: [ nil ])
				ifNil: [ result := 'unknown command <{1}>' format: { self argument } ] 
				ifNotNil: [ :command | result := command helpText ] ]
		ifFalse: [ 
			result := String streamContents: [ :out |
				out << 'help <command>'; cr.
				out << 'known commands are:'; cr.
				NeoConsoleCommand knownCommands 
					do: [ :each | 
						out space; space; << each prefix.
						NeoConsoleCommand defaultCommand = each ifTrue: [ out << ' DEFAULT' ] ] 
					separatedBy: [ out cr ] ] ]! !

!NeoConsoleHelpCommand methodsFor: 'initialization-release' stamp: 'SvenVanCaekenberghe 2/24/2015 16:48'!
on: input
	super on: input.
	arguments := self argument findTokens: Character separators.
	arguments := arguments allButFirst ! !

NeoConsoleCommand subclass: #NeoConsoleQuitCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleQuitCommand commentStamp: 'SvenVanCaekenberghe 2/24/2015 15:36' prior: 0!
Quit
!

!NeoConsoleQuitCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:36'!
prefix
	^ 'quit'! !

!NeoConsoleQuitCommand methodsFor: 'public' stamp: 'SvenVanCaekenberghe 2/24/2015 15:38'!
execute
	result := 'Bye!!'.
	^ nil! !

Object subclass: #NeoConsoleMetric
	instanceVariableNames: 'name description reader'
	classVariableNames: 'KnownMetrics'
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleMetric commentStamp: 'SvenVanCaekenberghe 2/24/2015 17:16' prior: 0!
I am NeoConsoleMetric. I represent a named metric with a description. I evaluate my reader to access my actual value.

My class side holds a list of #knownMetrics.!

!NeoConsoleMetric class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/16/2015 19:20'!
add: metric
	^ self knownMetrics 
		removeAllSuchThat: [ :each | each name = metric name ]; 
		add: metric! !

!NeoConsoleMetric class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 6/16/2015 19:20'!
initialize
	"self initialize"
	
	KnownMetrics := nil.
	self initializeMetrics! !

!NeoConsoleMetric class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 6/16/2015 17:06'!
initializeChronologyMetrics
	self add: (NeoConsoleMetric 
		named: 'system.date' 
		description: 'Current date' 
		reader: [ Date today printString ]).
	self add: (NeoConsoleMetric
		named: 'system.time'
		description: 'Current time'
		reader: [ Time now printString ]).
	self add: (NeoConsoleMetric
		named: 'system.timestamp'
		description: 'Current timestamp'
		reader: [ DateAndTime now printString ]).! !

!NeoConsoleMetric class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 6/16/2015 17:03'!
initializeMemoryMetrics
	self add: (NeoConsoleMetric 
		named: 'memory.total' 
		description: 'Total allocated memory' 
		reader: [ Smalltalk vm memoryEnd ]).
	self add: (NeoConsoleMetric
		named: 'memory.free'
		description: 'Free memory'
		reader: [ Smalltalk vm memoryEnd - Smalltalk vm youngSpaceEnd ])! !

!NeoConsoleMetric class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 6/16/2015 17:07'!
initializeMetrics
	self initializeMemoryMetrics.
	self initializeChronologyMetrics.
	self initializeProcessMetrics.! !

!NeoConsoleMetric class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 6/16/2015 18:25'!
initializeProcessMetrics
	self add: (NeoConsoleMetric 
		named: 'process.count' 
		description: 'Current process count' 
		reader: [ (Process allSubInstances reject: #isTerminated) size asString ]).
	self add: (NeoConsoleMetric
		named: 'process.list'
		description: 'Current list of processes'
		reader: [ ((Process allSubInstances reject: #isTerminated) collect: # name as: Array) printString ])! !

!NeoConsoleMetric class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:15'!
knownMetrics
	^ KnownMetrics ifNil: [ KnownMetrics := OrderedCollection new ]! !

!NeoConsoleMetric class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/16/2015 18:22'!
named: metricName
	^ self knownMetrics detect: [ :each | each name = metricName ] ifNone: [ NotFound signalFor: metricName ]! !

!NeoConsoleMetric class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/24/2015 17:18'!
named: metricName description: description reader: valueable
	^ self new
		name: metricName;
		description: description;
		reader: valueable;
		yourself! !

!NeoConsoleMetric methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:13'!
description
	^ description! !

!NeoConsoleMetric methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:14'!
description: string
	description := string! !

!NeoConsoleMetric methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:13'!
name
	^ name! !

!NeoConsoleMetric methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:14'!
name: string
	name := string! !

!NeoConsoleMetric methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/16/2015 18:23'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(; << name; nextPut: $)! !

!NeoConsoleMetric methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:13'!
reader
	^ reader! !

!NeoConsoleMetric methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:14'!
reader: aValuable
	reader := aValuable! !

!NeoConsoleMetric methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:14'!
value
	^ reader value! !

Object subclass: #NeoConsoleTelnetServer
	instanceVariableNames: 'process serverSocket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleTelnetServer commentStamp: 'SvenVanCaekenberghe 6/16/2015 19:23' prior: 0!
NeoConsoleTelnetServer offers a command line interface to a Pharo image.

Inspect:

  NeoConsoleTelnetServer new start.

In a terminal do:

  $ telnet localhost 4999

End with quit or an empty line.

Security warning: this service opens up your image for access to those with access to your local network (i.e. those logged in to your machine), without any further authenication, allowing them to do absolutely anything. Think and make sure that you know what you are doing.!

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 6/16/2015 16:17'!
executeRequestResponseLoopOn: stream
	| in out command executeResult |
	in := ZnCharacterReadStream on: stream.
	out := ZnCharacterWriteStream on: stream.
	self writeLine: ('Neo Console ', SystemVersion current asString) to: out.
	[
		stream nextPutAll: '> '; flush.
		command := (self readInputFrom: in).
		executeResult := [ command execute ] 
			on: Error 
			do: [ :exception | command printString, ' failed with ', exception printString ].
		self writeOutput: command result to: out.
		executeResult notNil	] whileTrue! !

!NeoConsoleTelnetServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/11/2015 16:09'!
initializeServerSocket
	serverSocket := Socket newTCP.
	serverSocket setOption: 'TCP_NODELAY' value: 1.
	serverSocket 
		listenOn: self port 
		backlogSize: 10 
		interface: NetNameResolver loopBackAddress.
	serverSocket isValid
		ifFalse: [ self error: 'Cannot create socket on port ' , self port printString ]! !

!NeoConsoleTelnetServer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/11/2015 15:49'!
isRunning
	"Return true when I am running"
	
	^ process notNil and: [ serverSocket notNil ]! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 2/11/2015 16:00'!
listenLoop
	"We create a listening Socket, then wait for a connection.
	After each connection we also check that the listening Socket is still valid 
	- if not we just make a recursive call to this method to start over."

	self initializeServerSocket.
	[ [ 
		serverSocket isValid 
			ifFalse: [
				"will trigger #ifCurtailed: block and destroy socket"
				^ self listenLoop ].
		self serveConnectionsOn: serverSocket ] repeat ]

		ifCurtailed: [ self releaseServerSocket ]! !

!NeoConsoleTelnetServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/11/2015 15:48'!
port
	^ 4999! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 2/24/2015 16:30'!
readInputFrom: stream
	| input lineReader line commandClass |
	lineReader := ZnFastLineReader on: stream.
	line := [ lineReader nextLine trimBoth ] on: ConnectionClosed, ConnectionTimedOut do: [ nil ].
	line isEmptyOrNil ifTrue: [ ^ NeoConsoleQuitCommand new ].
	commandClass := NeoConsoleCommand forLine: line.
	commandClass isSingleLine
		ifTrue: [ ^ commandClass new on: line; yourself ].
	input := String new writeStream.
	input nextPutAll: line; cr.
	[ 
		line := [ lineReader nextLine trimBoth ] on: ConnectionClosed, ConnectionTimedOut do: [ '' ].
		input nextPutAll: line; cr.
		line isEmpty ] whileFalse.
	^ commandClass new on: input contents; yourself! !

!NeoConsoleTelnetServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/11/2015 16:04'!
releaseServerSocket
	(Delay forMilliseconds: 10) wait.
	serverSocket destroy.
	serverSocket := nil! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 2/11/2015 16:12'!
serveConnectionsOn: listeningSocket
	"We wait up to acceptWaitTimeout seconds for an incoming connection.
	If we get one we wrap it in a SocketStream and #executeRequestResponseLoopOn: on it"
		 
	| stream socket |
	socket := listeningSocket waitForAcceptFor: 300.
	socket ifNil: [ ^ self ].
	stream := self socketStreamOn: socket.
	[ [ [ self executeRequestResponseLoopOn: stream ]
		ensure: [ [ stream close ] on: Exception do: [ ] ] ]
			ifCurtailed: [ socket destroy ] ]
				forkAt: Processor lowIOPriority
				named: self workerProcessName ! !

!NeoConsoleTelnetServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/11/2015 15:51'!
serverProcessName
	^ self class name asString! !

!NeoConsoleTelnetServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/11/2015 16:10'!
socketStreamOn: socket
	^ (ZdcSocketStream on: socket)
		binary;
		shouldSignal: true;
		autoFlush: false;
		timeout: self timeout;
		yourself! !

!NeoConsoleTelnetServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 2/11/2015 15:51'!
start
	self stop.
	process := [ [ self listenLoop ] repeat ] 
					forkAt: Processor highIOPriority 
					named: self serverProcessName! !

!NeoConsoleTelnetServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 2/11/2015 15:50'!
stop
	self isRunning ifFalse: [ ^ self ].
	process terminate. 
	process := nil.
! !

!NeoConsoleTelnetServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/16/2015 16:36'!
timeout
	^ 60! !

!NeoConsoleTelnetServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/11/2015 16:07'!
workerProcessName
	^ self serverProcessName, ' Connection Handler'! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 2/24/2015 16:23'!
writeLine: string to: stream
	stream nextPutAll: string; crlf; flush! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 2/24/2015 16:23'!
writeOutput: string to: stream
	string lines
		do: [ :each | stream nextPutAll: each ]
		separatedBy: [ stream crlf ].
	stream crlf; flush! !
NeoConsoleMetric initialize!
